@@ -1044,7 +1044,11 @@ object_instance_trace(JSTracer *tracer,
     ObjectInstance *priv;
     GList *iter;
 
-    priv = priv_from_js(tracer->context, obj);
+    /* DO NOT use priv_from_js here: that uses JS_BeginRequest,
+       but this is called from the GC thread, and deadlocks
+       We know we're of the right JSClass anyway.
+    */
+    priv = JS_GetPrivate(tracer->context, obj);
 
     for (iter = priv->signals; iter; iter = iter->next) {
         ConnectData *cd = iter->data;
@@ -1388,11 +1392,13 @@ emit_func(JSContext *context,
 
     if (signal_query.return_type != G_TYPE_NONE) {
         if (!gjs_value_from_g_value(context,
-                                       &retval,
-                                       &rvalue))
+                                    &retval,
+                                    &rvalue))
             failed = TRUE;
 
         g_value_unset(&rvalue);
+    } else {
+        retval = JSVAL_VOID;
     }
 
     for (i = 0; i < (signal_query.n_params + 1); ++i) {
